@@ -123,21 +123,24 @@ individual_can_audio_video_call (FolksIndividual *individual,
     gboolean *can_audio_call,
     gboolean *can_video_call)
 {
-  GList *personas, *l;
+  GeeSet *personas;
+  GeeIterator *iter;
   gboolean can_audio = FALSE, can_video = FALSE;
 
   personas = folks_individual_get_personas (individual);
-  for (l = personas; l != NULL; l = l->next)
+  iter = gee_iterable_iterator (GEE_ITERABLE (personas));
+  while (gee_iterator_next (iter))
     {
+      FolksPersona *persona = gee_iterator_get (iter);
       TpContact *tp_contact;
       EmpathyContact *contact;
 
-      if (!empathy_folks_persona_is_interesting (FOLKS_PERSONA (l->data)))
-        continue;
+      if (!empathy_folks_persona_is_interesting (persona))
+        goto while_finish;
 
-      tp_contact = tpf_persona_get_contact (TPF_PERSONA (l->data));
+      tp_contact = tpf_persona_get_contact (TPF_PERSONA (persona));
       contact = empathy_contact_dup_from_tp_contact (tp_contact);
-      empathy_contact_set_persona (contact, FOLKS_PERSONA (l->data));
+      empathy_contact_set_persona (contact, persona);
 
       can_audio = can_audio || empathy_contact_get_capabilities (contact) &
           EMPATHY_CAPABILITIES_AUDIO;
@@ -145,10 +148,13 @@ individual_can_audio_video_call (FolksIndividual *individual,
           EMPATHY_CAPABILITIES_VIDEO;
 
       g_object_unref (contact);
+while_finish:
+      g_clear_object (&persona);
 
       if (can_audio && can_video)
         break;
     }
+  g_clear_object (&iter);
 
   *can_audio_call = can_audio;
   *can_video_call = can_video;
@@ -157,20 +163,23 @@ individual_can_audio_video_call (FolksIndividual *individual,
 static const gchar * const *
 individual_get_client_types (FolksIndividual *individual)
 {
-  GList *personas, *l;
+  GeeSet *personas;
+  GeeIterator *iter;
   const gchar * const *types = NULL;
   FolksPresenceType presence_type = FOLKS_PRESENCE_TYPE_UNSET;
 
   personas = folks_individual_get_personas (individual);
-  for (l = personas; l != NULL; l = l->next)
+  iter = gee_iterable_iterator (GEE_ITERABLE (personas));
+  while (gee_iterator_next (iter))
     {
       FolksPresenceDetails *presence;
+      FolksPersona *persona = gee_iterator_get (iter);
 
-      /* We only want personas which implement FolksPresenceDetails */
-      if (!FOLKS_IS_PRESENCE_DETAILS (l->data))
-        continue;
+      /* We only want personas which have presence and a TpContact */
+      if (!empathy_folks_persona_is_interesting (persona))
+        goto while_finish;
 
-      presence = FOLKS_PRESENCE_DETAILS (l->data);
+      presence = FOLKS_PRESENCE_DETAILS (persona);
 
       if (folks_presence_details_typecmp (
               folks_presence_details_get_presence_type (presence),
@@ -180,10 +189,14 @@ individual_get_client_types (FolksIndividual *individual)
 
           presence_type = folks_presence_details_get_presence_type (presence);
 
-          tp_contact = tpf_persona_get_contact (TPF_PERSONA (l->data));
+          tp_contact = tpf_persona_get_contact (TPF_PERSONA (persona));
           types = tp_contact_get_client_types (tp_contact);
         }
+
+while_finish:
+      g_clear_object (&persona);
     }
+  g_clear_object (&iter);
 
   return types;
 }
@@ -405,11 +418,7 @@ individual_store_add_individual (EmpathyIndividualStore *self,
 {
   EmpathyIndividualStorePriv *priv;
   GtkTreeIter iter;
-  GHashTable *group_set = NULL;
-  GList *groups = NULL, *l;
-  EmpathyContact *contact;
-  TpConnection *connection;
-  gchar *protocol_name;
+  GeeIterator *group_iter = NULL;
 
   priv = GET_PRIV (self);
 
@@ -419,22 +428,32 @@ individual_store_add_individual (EmpathyIndividualStore *self,
 
   if (priv->show_groups)
     {
+      GeeSet *group_set = NULL;
+
       group_set = folks_group_details_get_groups (
           FOLKS_GROUP_DETAILS (individual));
-      groups = g_hash_table_get_keys (group_set);
+
+      if (gee_collection_get_size (GEE_COLLECTION (group_set)) > 0)
+        group_iter = gee_iterable_iterator (GEE_ITERABLE (group_set));
     }
 
-  contact = empathy_contact_dup_from_folks_individual (individual);
-  connection = empathy_contact_get_connection (contact);
-
-  tp_connection_parse_object_path (connection, &protocol_name, NULL);
-
-  if (groups == NULL)
+  /* fall-back groups, in case there are no named groups */
+  if (group_iter == NULL)
     {
       GtkTreeIter iter_group, *parent;
+      EmpathyContact *contact;
+      TpConnection *connection;
+      gchar *protocol_name = NULL;
 
       parent = &iter_group;
 
+      contact = empathy_contact_dup_from_folks_individual (individual);
+      if (contact != NULL)
+        {
+          connection = empathy_contact_get_connection (contact);
+          tp_connection_parse_object_path (connection, &protocol_name, NULL);
+        }
+
       if (!priv->show_groups)
         parent = NULL;
       else if (!tp_strdiff (protocol_name, "local-xmpp"))
@@ -453,22 +472,26 @@ individual_store_add_individual (EmpathyIndividualStore *self,
 
       add_individual_to_store (GTK_TREE_STORE (self), &iter, parent,
           individual);
+
+      g_free (protocol_name);
+      g_clear_object (&contact);
     }
 
-  g_free (protocol_name);
-
   /* Else add to each group. */
-  for (l = groups; l; l = l->next)
+  while (group_iter != NULL && gee_iterator_next (group_iter))
     {
+      gchar *group_name = gee_iterator_get (group_iter);
       GtkTreeIter iter_group;
 
-      individual_store_get_group (self, l->data, &iter_group, NULL, NULL,
+      individual_store_get_group (self, group_name, &iter_group, NULL, NULL,
           FALSE);
 
       add_individual_to_store (GTK_TREE_STORE (self), &iter, &iter_group,
           individual);
+
+      g_free (group_name);
     }
-  g_list_free (groups);
+  g_clear_object (&group_iter);
 
   if (priv->show_groups &&
       folks_favourite_details_get_is_favourite (
@@ -485,8 +508,6 @@ individual_store_add_individual (EmpathyIndividualStore *self,
     }
 
   individual_store_contact_update (self, individual);
-
-  tp_clear_object (&contact);
 }
 
 static void
@@ -858,62 +879,74 @@ individual_store_contact_updated_cb (EmpathyContact *contact,
 
 static void
 individual_personas_changed_cb (FolksIndividual *individual,
-    GList *added,
-    GList *removed,
+    GeeSet *added,
+    GeeSet *removed,
     EmpathyIndividualStore *self)
 {
-  GList *l;
+  GeeIterator *iter;
 
   DEBUG ("Individual '%s' personas-changed.",
       folks_individual_get_id (individual));
 
+  iter = gee_iterable_iterator (GEE_ITERABLE (removed));
   /* FIXME: libfolks hasn't grown capabilities support yet, so we have to go
    * through the EmpathyContacts for them. */
-  for (l = removed; l != NULL; l = l->next)
+  while (gee_iterator_next (iter))
     {
+      TpfPersona *persona = gee_iterator_get (iter);
       TpContact *tp_contact;
       EmpathyContact *contact;
 
-      if (!TPF_IS_PERSONA (l->data))
-        continue;
+      if (TPF_IS_PERSONA (persona))
+        {
+          tp_contact = tpf_persona_get_contact (persona);
+          contact = empathy_contact_dup_from_tp_contact (tp_contact);
+          empathy_contact_set_persona (contact, FOLKS_PERSONA (persona));
 
-      tp_contact = tpf_persona_get_contact (TPF_PERSONA (l->data));
-      contact = empathy_contact_dup_from_tp_contact (tp_contact);
-      empathy_contact_set_persona (contact, FOLKS_PERSONA (l->data));
+          g_object_set_data (G_OBJECT (contact), "individual", NULL);
+          g_signal_handlers_disconnect_by_func (contact,
+              (GCallback) individual_store_contact_updated_cb, self);
 
-      g_object_set_data (G_OBJECT (contact), "individual", NULL);
-      g_signal_handlers_disconnect_by_func (contact,
-          (GCallback) individual_store_contact_updated_cb, self);
+          g_object_unref (contact);
+        }
 
-      g_object_unref (contact);
+      g_clear_object (&persona);
     }
+  g_clear_object (&iter);
 
-  for (l = added; l != NULL; l = l->next)
+  iter = gee_iterable_iterator (GEE_ITERABLE (added));
+  while (gee_iterator_next (iter))
     {
+      TpfPersona *persona = gee_iterator_get (iter);
       TpContact *tp_contact;
       EmpathyContact *contact;
 
-      if (!TPF_IS_PERSONA (l->data))
-        continue;
+      if (TPF_IS_PERSONA (persona))
+        {
+          tp_contact = tpf_persona_get_contact (persona);
+          contact = empathy_contact_dup_from_tp_contact (tp_contact);
+          empathy_contact_set_persona (contact, FOLKS_PERSONA (persona));
 
-      tp_contact = tpf_persona_get_contact (TPF_PERSONA (l->data));
-      contact = empathy_contact_dup_from_tp_contact (tp_contact);
-      empathy_contact_set_persona (contact, FOLKS_PERSONA (l->data));
+          g_object_set_data (G_OBJECT (contact), "individual", individual);
+          g_signal_connect (contact, "notify::capabilities",
+              (GCallback) individual_store_contact_updated_cb, self);
+          g_signal_connect (contact, "notify::client-types",
+              (GCallback) individual_store_contact_updated_cb, self);
 
-      g_object_set_data (G_OBJECT (contact), "individual", individual);
-      g_signal_connect (contact, "notify::capabilities",
-          (GCallback) individual_store_contact_updated_cb, self);
-      g_signal_connect (contact, "notify::client-types",
-          (GCallback) individual_store_contact_updated_cb, self);
+          g_object_unref (contact);
+        }
 
-      g_object_unref (contact);
+      g_clear_object (&persona);
     }
+  g_clear_object (&iter);
 }
 
-static void
+void
 individual_store_add_individual_and_connect (EmpathyIndividualStore *self,
     FolksIndividual *individual)
 {
+  GeeSet *empty_set = gee_set_empty (G_TYPE_NONE, NULL, NULL);
+
   individual_store_add_individual (self, individual);
 
   g_signal_connect (individual, "notify::avatar",
@@ -927,16 +960,22 @@ individual_store_add_individual_and_connect (EmpathyIndividualStore *self,
   g_signal_connect (individual, "personas-changed",
       (GCallback) individual_personas_changed_cb, self);
 
+  /* provide an empty set so the callback can assume non-NULL sets */
   individual_personas_changed_cb (individual,
-      folks_individual_get_personas (individual), NULL, self);
+      folks_individual_get_personas (individual), empty_set, self);
+  g_clear_object (&empty_set);
 }
 
 static void
 individual_store_disconnect_individual (EmpathyIndividualStore *self,
     FolksIndividual *individual)
 {
-  individual_personas_changed_cb (individual, NULL,
+  GeeSet *empty_set = gee_set_empty (G_TYPE_NONE, NULL, NULL);
+
+  /* provide an empty set so the callback can assume non-NULL sets */
+  individual_personas_changed_cb (individual, empty_set,
       folks_individual_get_personas (individual), self);
+  g_clear_object (&empty_set);
 
   g_signal_handlers_disconnect_by_func (individual,
       (GCallback) individual_store_individual_updated_cb, self);
@@ -944,7 +983,7 @@ individual_store_disconnect_individual (EmpathyIndividualStore *self,
       (GCallback) individual_personas_changed_cb, self);
 }
 
-static void
+void
 individual_store_remove_individual_and_disconnect (
     EmpathyIndividualStore *self,
     FolksIndividual *individual)
@@ -1533,6 +1572,7 @@ individual_store_setup (EmpathyIndividualStore *self)
     G_TYPE_BOOLEAN,             /* Can make video calls */
     G_TYPE_BOOLEAN,             /* Is a fake group */
     G_TYPE_STRV,                /* Client types */
+    G_TYPE_UINT,                /* Event count */
   };
 
   priv = GET_PRIV (self);
@@ -1925,22 +1965,28 @@ individual_store_get_individual_status_icon_with_icon_name (
   EmpathyIndividualStorePriv *priv;
   const gchar *protocol_name = NULL;
   gchar *icon_name = NULL;
-  GList *personas, *l;
-  guint contact_count;
+  GeeSet *personas;
+  GeeIterator *iter;
+  guint contact_count = 0;
   EmpathyContact *contact = NULL;
   gboolean show_protocols_here;
 
   priv = GET_PRIV (self);
 
   personas = folks_individual_get_personas (individual);
-  for (l = personas, contact_count = 0; l; l = l->next)
+  iter = gee_iterable_iterator (GEE_ITERABLE (personas));
+  while (gee_iterator_next (iter))
     {
-      if (empathy_folks_persona_is_interesting (FOLKS_PERSONA (l->data)))
+      FolksPersona *persona = gee_iterator_get (iter);
+      if (empathy_folks_persona_is_interesting (persona))
         contact_count++;
 
+      g_clear_object (&persona);
+
       if (contact_count > 1)
         break;
     }
+  g_clear_object (&iter);
 
   show_protocols_here = (priv->show_protocols && (contact_count == 1));
   if (show_protocols_here)
