@@ -37,6 +37,7 @@
 
 #include "empathy-tp-chat.h"
 #include "empathy-chatroom-manager.h"
+#include "empathy-channel-factory.h"
 #include "empathy-utils.h"
 
 #define DEBUG_FLAG EMPATHY_DEBUG_OTHER
@@ -192,7 +193,16 @@ add_chatroom (EmpathyChatroomManager *self,
 
   priv->chatrooms = g_list_prepend (priv->chatrooms, g_object_ref (chatroom));
 
-  g_signal_connect (chatroom, "notify",
+  /* Watch only those properties which are exported in the save file */
+  g_signal_connect (chatroom, "notify::name",
+      G_CALLBACK (chatroom_changed_cb), self);
+  g_signal_connect (chatroom, "notify::room",
+      G_CALLBACK (chatroom_changed_cb), self);
+  g_signal_connect (chatroom, "notify::account",
+      G_CALLBACK (chatroom_changed_cb), self);
+  g_signal_connect (chatroom, "notify::auto-connect",
+      G_CALLBACK (chatroom_changed_cb), self);
+  g_signal_connect (chatroom, "notify::always_urgent",
       G_CALLBACK (chatroom_changed_cb), self);
 }
 
@@ -414,9 +424,17 @@ static void
 clear_chatrooms (EmpathyChatroomManager *self)
 {
   EmpathyChatroomManagerPriv *priv = GET_PRIV (self);
-  GList *l;
+  GList *l, *tmp;
 
-  for (l = priv->chatrooms; l != NULL; l = g_list_next (l))
+  tmp = priv->chatrooms;
+
+  /* Unreffing the chatroom may result in destroying the underlying
+   * EmpathyTpChat which will fire the invalidated signal and so make us
+   * re-call this function. We already set priv->chatrooms to NULL so we won't
+   * try to destroy twice the same objects. */
+  priv->chatrooms = NULL;
+
+  for (l = tmp; l != NULL; l = g_list_next (l))
     {
       EmpathyChatroom *chatroom = l->data;
 
@@ -427,8 +445,7 @@ clear_chatrooms (EmpathyChatroomManager *self)
       g_object_unref (chatroom);
     }
 
-  g_list_free (priv->chatrooms);
-  priv->chatrooms = NULL;
+  g_list_free (tmp);
 }
 
 static void
@@ -529,6 +546,7 @@ empathy_chatroom_manager_constructor (GType type,
   EmpathyChatroomManagerPriv *priv;
   GError *error = NULL;
   TpDBusDaemon *dbus;
+  EmpathyChannelFactory *factory;
 
   if (chatroom_manager_singleton != NULL)
     return g_object_ref (chatroom_manager_singleton);
@@ -588,6 +606,11 @@ empathy_chatroom_manager_constructor (GType type,
   tp_base_client_add_connection_features_varargs (priv->observer,
       TP_CONNECTION_FEATURE_CAPABILITIES, NULL);
 
+  factory = empathy_channel_factory_dup ();
+
+  tp_base_client_set_channel_factory (priv->observer,
+      TP_CLIENT_CHANNEL_FACTORY (factory));
+
   if (!tp_base_client_register (priv->observer, &error))
     {
       g_critical ("Failed to register Observer: %s", error->message);
@@ -595,6 +618,7 @@ empathy_chatroom_manager_constructor (GType type,
       g_error_free (error);
     }
 
+  g_object_unref (factory);
   return obj;
 }
 
@@ -669,13 +693,9 @@ gboolean
 empathy_chatroom_manager_add (EmpathyChatroomManager *manager,
     EmpathyChatroom *chatroom)
 {
-  EmpathyChatroomManagerPriv *priv;
-
   g_return_val_if_fail (EMPATHY_IS_CHATROOM_MANAGER (manager), FALSE);
   g_return_val_if_fail (EMPATHY_IS_CHATROOM (chatroom), FALSE);
 
-  priv = GET_PRIV (manager);
-
   /* don't add more than once */
   if (!empathy_chatroom_manager_find (manager,
       empathy_chatroom_get_account (chatroom),
@@ -827,7 +847,10 @@ empathy_chatroom_manager_get_chatrooms (EmpathyChatroomManager *manager,
 }
 
 static void
-chatroom_manager_chat_destroyed_cb (EmpathyTpChat *chat,
+chatroom_manager_chat_invalidated_cb (EmpathyTpChat *chat,
+  guint domain,
+  gint code,
+  gchar *message,
   gpointer manager)
 {
   EmpathyChatroomManagerPriv *priv = GET_PRIV (manager);
@@ -869,15 +892,16 @@ observe_channels_cb (TpSimpleObserver *observer,
 
   for (l = channels; l != NULL; l = g_list_next (l))
     {
-      TpChannel *channel = l->data;
-      EmpathyTpChat *tp_chat;
+      EmpathyTpChat *tp_chat = l->data;
       const gchar *roomname;
       EmpathyChatroom *chatroom;
 
-      if (tp_proxy_get_invalidated (channel) != NULL)
+      if (tp_proxy_get_invalidated ((TpChannel *) tp_chat) != NULL)
+        continue;
+
+      if (!EMPATHY_IS_TP_CHAT (tp_chat))
         continue;
 
-      tp_chat = empathy_tp_chat_new (account, channel);
       roomname = empathy_tp_chat_get_id (tp_chat);
       chatroom = empathy_chatroom_manager_find (self, account, roomname);
 
@@ -890,12 +914,9 @@ observe_channels_cb (TpSimpleObserver *observer,
         }
 
       empathy_chatroom_set_tp_chat (chatroom, tp_chat);
-      g_object_unref (tp_chat);
 
-      /* A TpChat is always destroyed as it only gets unreffed after the channel
-       * has been invalidated in the dispatcher..  */
-      g_signal_connect (tp_chat, "destroy",
-        G_CALLBACK (chatroom_manager_chat_destroyed_cb),
+      g_signal_connect (tp_chat, "invalidated",
+        G_CALLBACK (chatroom_manager_chat_invalidated_cb),
         self);
     }
 
