@@ -30,11 +30,13 @@
 
 #include <telepathy-farstream/telepathy-farstream.h>
 
+#include <libempathy/empathy-channel-factory.h>
 #include <libempathy/empathy-utils.h>
 #include <libempathy/empathy-tp-contact-factory.h>
 
+#include <libempathy-gtk/empathy-call-utils.h>
+
 #include "empathy-call-handler.h"
-#include "empathy-call-factory.h"
 #include "src-marshal.h"
 
 #define DEBUG_FLAG EMPATHY_DEBUG_VOIP
@@ -76,7 +78,7 @@ enum {
 
 /* private structure */
 
-typedef struct {
+struct _EmpathyCallHandlerPriv {
   TpyCallChannel *call;
 
   EmpathyContact *contact;
@@ -94,7 +96,7 @@ typedef struct {
   FsCandidate *video_remote_candidate;
   FsCandidate *audio_local_candidate;
   FsCandidate *video_local_candidate;
-} EmpathyCallHandlerPriv;
+};
 
 #define GET_PRIV(obj) EMPATHY_GET_PRIV (obj, EmpathyCallHandler)
 
@@ -167,6 +169,25 @@ on_get_contacts_cb (TpConnection *connection,
   g_object_notify (G_OBJECT (self), "members");
 }
 
+static void
+on_call_invalidated_cb (TpyCallChannel *call,
+    guint domain,
+    gint code,
+    gchar *message,
+    EmpathyCallHandler *self)
+{
+  EmpathyCallHandlerPriv *priv = self->priv;
+
+  if (priv->call == call)
+    {
+      /* Invalidated unexpectedly? Fake call ending */
+      g_signal_emit (self, signals[STATE_CHANGED], 0,
+          TPY_CALL_STATE_ENDED, NULL);
+      tp_clear_object (&priv->call);
+      tp_clear_object (&priv->tfchannel);
+    }
+}
+
 static void
 on_call_state_changed_cb (TpyCallChannel *call,
   TpyCallState state,
@@ -175,10 +196,17 @@ on_call_state_changed_cb (TpyCallChannel *call,
   GHashTable *call_state_details,
   EmpathyCallHandler *handler)
 {
-  if (state == TPY_CALL_STATE_ENDED)
-    tp_channel_close_async (TP_CHANNEL (call), NULL, NULL);
+  EmpathyCallHandlerPriv *priv = handler->priv;
 
   g_signal_emit (handler, signals[STATE_CHANGED], 0, state);
+
+  if (state == TPY_CALL_STATE_ENDED)
+    {
+      tp_channel_close_async (TP_CHANNEL (call), NULL, NULL);
+
+      tp_clear_object (&priv->call);
+      tp_clear_object (&priv->tfchannel);
+    }
 }
 
 static void
@@ -254,6 +282,8 @@ empathy_call_handler_set_property (GObject *object,
 
         tp_g_signal_connect_object (priv->call, "state-changed",
           G_CALLBACK (on_call_state_changed_cb), object, 0);
+        tp_g_signal_connect_object (priv->call, "invalidated",
+          G_CALLBACK (on_call_invalidated_cb), object, 0);
         break;
       case PROP_INITIAL_AUDIO:
         priv->initial_audio = g_value_get_boolean (value);
@@ -626,7 +656,7 @@ empathy_call_handler_bus_message (EmpathyCallHandler *handler,
       FsCodec *codec;
       FsSession *session;
 
-      g_print ("empathy_call_handler_bus_message: farsight-send-codec-changed\n");
+      DEBUG ("farsight-send-codec-changed");
 
       val = gst_structure_get_value (s, "codec");
       codec = g_value_get_boxed (val);
@@ -643,7 +673,7 @@ empathy_call_handler_bus_message (EmpathyCallHandler *handler,
       GList *codecs;
       FsStream *stream;
 
-      g_print ("empathy_call_handler_bus_message: farsight-recv-codecs-changed\n");
+      DEBUG ("farsight-recv-codecs-changed");
 
       val = gst_structure_get_value (s, "codecs");
       codecs = g_value_get_boxed (val);
@@ -660,7 +690,7 @@ empathy_call_handler_bus_message (EmpathyCallHandler *handler,
       FsCandidate *remote_candidate, *local_candidate;
       FsStream *stream;
 
-      g_print ("empathy_call_handler_bus_message: farsight-new-active-candidate-pair\n");
+      DEBUG ("farsight-new-active-candidate-pair");
 
       val = gst_structure_get_value (s, "remote-candidate");
       remote_candidate = g_value_get_boxed (val);
@@ -791,7 +821,7 @@ on_tf_channel_content_removed_cb (TfChannel *tfchannel,
   GstPad *spad;
   gboolean retval;
 
-  g_print ("removing content\n");
+  DEBUG ("removing content");
 
   g_object_get (content, "media-type", &mtype,
     "sink-pad", &spad, NULL);
@@ -856,7 +886,21 @@ empathy_call_handler_start_tpfs (EmpathyCallHandler *self)
       on_tf_channel_ready, self);
 }
 
-#if 0
+static void
+on_call_accepted_cb (GObject *source_object,
+    GAsyncResult *res,
+    gpointer user_data)
+{
+  TpyCallChannel *call = TPY_CALL_CHANNEL (source_object);
+  GError *error = NULL;
+
+  if (!tpy_call_channel_accept_finish (call, res, &error))
+    {
+      g_warning ("could not accept Call: %s", error->message);
+      g_error_free (error);
+    }
+}
+
 static void
 empathy_call_handler_request_cb (GObject *source,
     GAsyncResult *result,
@@ -884,26 +928,15 @@ empathy_call_handler_request_cb (GObject *source,
     }
 
   priv->call = TPY_CALL_CHANNEL (channel);
+  tp_g_signal_connect_object (priv->call, "state-changed",
+    G_CALLBACK (on_call_state_changed_cb), self, 0);
+  tp_g_signal_connect_object (priv->call, "invalidated",
+    G_CALLBACK (on_call_invalidated_cb), self, 0);
 
   g_object_notify (G_OBJECT (self), "call-channel");
 
   empathy_call_handler_start_tpfs (self);
-}
-#endif
-
-static void
-on_call_accepted_cb (GObject *source_object,
-    GAsyncResult *res,
-    gpointer user_data)
-{
-  TpyCallChannel *call = TPY_CALL_CHANNEL (source_object);
-  GError *error = NULL;
-
-  if (!tpy_call_channel_accept_finish (call, res, &error))
-    {
-      g_warning ("could not accept Call: %s", error->message);
-      g_error_free (error);
-    }
+  tpy_call_channel_accept_async (priv->call, on_call_accepted_cb, NULL);
 }
 
 void
@@ -911,9 +944,10 @@ empathy_call_handler_start_call (EmpathyCallHandler *handler,
     gint64 timestamp)
 {
   EmpathyCallHandlerPriv *priv = GET_PRIV (handler);
-/*TpAccountChannelRequest *req;
+  EmpathyChannelFactory *channel_factory;
+  TpAccountChannelRequest *req;
   TpAccount *account;
-  GHashTable *request;*/
+  GHashTable *request;
 
   if (priv->call != NULL)
     {
@@ -921,27 +955,27 @@ empathy_call_handler_start_call (EmpathyCallHandler *handler,
       tpy_call_channel_accept_async (priv->call, on_call_accepted_cb, NULL);
       return;
     }
-  else
-    {
-      g_warning ("No Call channel!");
-    }
 
-#if 0
   /* No TpyCallChannel (we are redialing). Request a new call channel */
   g_assert (priv->contact != NULL);
 
   account = empathy_contact_get_account (priv->contact);
-  request = empathy_call_create_call_request (priv->contact,
+  request = empathy_call_create_call_request (
+      empathy_contact_get_id (priv->contact),
       priv->initial_audio, priv->initial_video);
 
   req = tp_account_channel_request_new (account, request, timestamp);
 
+  channel_factory = empathy_channel_factory_dup ();
+  tp_account_channel_request_set_channel_factory (req,
+      TP_CLIENT_CHANNEL_FACTORY (channel_factory));
+  g_object_unref (channel_factory);
+
   tp_account_channel_request_create_and_handle_channel_async (req, NULL,
       empathy_call_handler_request_cb, handler);
 
   g_object_unref (req);
   g_hash_table_unref (request);
-#endif
 }
 
 /**
